/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_NDBasic.c - automatic nested dissection by A. George
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/


#include "morsy_NDBasic.h"

void BasicNestedDissectionProcedure(int* GraphXadj, int* GraphAdjncy, 
                                    int n, int* Separators)
{
  int num;
  int vertex;
  int nSep;
  int sepPos;
  int subgrPos;
  int nSubgr;
  int newSubgr;
  int sepLevel;
  int nLevels;
  int i;
  int* LevelStructure; 
  int* LevelIndex; 
  int* VertexMarks; 
  int* Subgraphs;

  VertexMarks = (int*)malloc(sizeof(int)*n);
  LevelStructure = (int*)malloc(sizeof(int)*n);
  LevelIndex = (int*)malloc(sizeof(int)*(n + 1));
  Subgraphs = (int*)malloc(sizeof(int)*n);
  
  for (i = 0; i < n; i++)
	  VertexMarks[i] = 1;

  num = n; 
  sepPos = 0; 
  Subgraphs[0] = 0;
  subgrPos = 0; 
  nSubgr = 1; 

  while (num > 0)
  {
    vertex = Subgraphs[subgrPos];
    sepLevel = FindSeparator(vertex, GraphAdjncy, GraphXadj, VertexMarks,
      LevelStructure, LevelIndex, &Separators, sepPos, &nSep, &nLevels);

    for (i = 0; i < nSep; i++)
      VertexMarks[Separators[sepPos + i]] = 0;

    newSubgr = GetNewSubgraphs(&Subgraphs, LevelStructure, 
      LevelIndex, sepLevel, nSubgr - 1, nLevels);

    num -= nSep; sepPos += nSep; nSubgr += newSubgr; subgrPos++;
    if ((nSubgr == subgrPos)&&(num > 0))
    {
      Subgraphs[nSubgr] = LinearSearch(VertexMarks, 1, n);
      nSubgr++;
    }     
  }

  free(VertexMarks);
  free(LevelStructure);
  free(LevelIndex);
  free(Subgraphs);
}
