/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_connectedComponents.c - graph connected components search
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/


#include "morsy_connectedComponents.h"

void ConnectedComponentsFromPartition(DisjointSetList* DisjointSet,
                                      GraphStructure* Graph,
                                      int* partition)
{
	int i;
  int j;

	MakeAllDisjointSets(DisjointSet);

  for (i = 0; i < Graph->nVertexes; i++)
  {
    if (partition[i] != pSeparator)
    {
      for (j = Graph->Xadj[i]; j < Graph->Xadj[i + 1]; j++)
      {	
        if (partition[Graph->Adjncy[j]] != pSeparator)
        {
          UnionDisjointSets(DisjointSet, i, Graph->Adjncy[j]);
        }
      }
    }
  }
}

void ConnectedComponents(DisjointSetList* DisjointSet,
                         int* Xadj, int* Adjncy, int nVertexes)
{
  int i;
  int j;

  MakeAllDisjointSets(DisjointSet);

  for (i = 0; i < nVertexes; i++)
  {
    for (j = Xadj[i]; j < Xadj[i + 1]; j++)
    {	
      UnionDisjointSets(DisjointSet, i, Adjncy[j]);
    }
  }
}

int CreateComponentsMap(DisjointSetList* graphComponents, int* partition, 
                        int* map, int n, int* componentsHeads)
{
  int nComponents = 0;
  int head;
  int headPosInComponents;
  int i;
  int* first = (int*)malloc(sizeof(int) * n);
  int* prev = (int*)malloc(sizeof(int) * n);
  int* headsPos = (int*)malloc(sizeof(int) * n);
  int* mapPos = (int*)malloc(sizeof(int) * n);

  memset(mapPos, 0, sizeof(int)*n);

  for(i = 0; i < n; i++)
  {
    map[i] = -1;
    headsPos[i] = -1;
  }

  for(i = 0; i < n; i++)
  {
    if (partition[i] != pSeparator)
    {
      head = graphComponents->head[i];
      if (headsPos[head] == -1)
      {
        componentsHeads[nComponents] = head;
        headsPos[head] = nComponents;
        first[nComponents] = i;
        nComponents++;
      }
      else
      {
        graphComponents->next[prev[headsPos[head]]] = i;
      }
      headPosInComponents = headsPos[head];
      map[i] = mapPos[headPosInComponents];
      mapPos[headPosInComponents]++;

      prev[headPosInComponents] = i;
    }
  }
  for(i = 0; i < nComponents; i++)
  {
    graphComponents->next[prev[i]] = -1;
    graphComponents->setSize[first[i]] = graphComponents->setSize[componentsHeads[i]];
    componentsHeads[i] = first[i];
  }

  free(headsPos);
  free(mapPos);
  free(prev);
  free(first);

  return nComponents;
}

int CreateGraphComponentsMap(DisjointSetList* graphComponents,
                             ConnectedComponentsStructure* CC)
{
  int nComponents = 0;
  int head;
  int headPosInComponents;
  int i;
  int n = CC->nAllVertexes;
  int* map = CC->map;
  int* componentsHeads = CC->heads;
  int* first = (int*)malloc(sizeof(int) * n);
  int* prev = (int*)malloc(sizeof(int) * n);
  int* headsPos = (int*)malloc(sizeof(int) * n);
  int* mapPos = (int*)malloc(sizeof(int) * n);

  memset(mapPos, 0, sizeof(int)*n);

  for(i = 0; i < n; i++)
  {
    map[i] = -1;
    headsPos[i] = -1;
  }

  for(i = 0; i < n; i++)
  {
    head = graphComponents->head[i];
    if (headsPos[head] == -1)
    {
      componentsHeads[nComponents] = head;
      headsPos[head] = nComponents;
      first[nComponents] = i;
      nComponents++;
    }
    else
    {
      graphComponents->next[prev[headsPos[head]]] = i;
    }
    headPosInComponents = headsPos[head];
    map[i] = mapPos[headPosInComponents];
    mapPos[headPosInComponents]++;

    prev[headPosInComponents] = i;
  }
  for(i = 0; i < nComponents; i++)
  {
    graphComponents->next[prev[i]] = -1;
    graphComponents->setSize[first[i]] = graphComponents->setSize[componentsHeads[i]];
    componentsHeads[i] = first[i];
  }

  free(headsPos);
  free(mapPos);
  free(prev);
  free(first);

  return nComponents;
}
