/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_disjointSets.c - operations with the disjoint sets data structure
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*
*/


#include "morsy_disjointSets.h"

#include <stdlib.h>


//-----------------------------------------------------------------------------
//disjoint set's functions
//-----------------------------------------------------------------------------

void InitializeDisjointSets(DisjointSetList* DisjointSets, int n)
{
  DisjointSets->head = (int*)malloc(sizeof(int)*n);
  DisjointSets->next = (int*)malloc(sizeof(int)*n);
  DisjointSets->setSize = (int*)malloc(sizeof(int)*n);
  DisjointSets->n = n;
}

void MakeAllDisjointSets(DisjointSetList* DisjointSets)
{
  int i;

  for (i = 0; i < DisjointSets->n; i++)
  {
    DisjointSets->head[i] = i;
    DisjointSets->next[i] = -1;
    DisjointSets->setSize[i] = 1;
  }
}

void InsertList(DisjointSetList* DisjointSets, int small, int big)
{
  int oldNext;
  int j;
  int i;

  oldNext = DisjointSets->next[big];
  DisjointSets->next[big] = small;

  j = small;
  for (i = 0; i < DisjointSets->setSize[small] - 1; i++)
  {
    DisjointSets->head[j] = big;
    j = DisjointSets->next[j];
  }

  DisjointSets->head[j] = big;
  DisjointSets->next[j] = oldNext;
  DisjointSets->setSize[big] += DisjointSets->setSize[small];
}

void UnionDisjointSets(DisjointSetList* DisjointSets, int x, int y)
{
  if (DisjointSets->head[x] == DisjointSets->head[y])
    return;

  if (DisjointSets->setSize[DisjointSets->head[x]] > 
      DisjointSets->setSize[DisjointSets->head[y]])
  {
    InsertList(DisjointSets, DisjointSets->head[y], DisjointSets->head[x]);
  }
  else 
  {
    InsertList(DisjointSets, DisjointSets->head[x], DisjointSets->head[y]);
  }
}

void FreeDisjointSet(DisjointSetList* DisjointSets)
{
  free(DisjointSets->head);
  free(DisjointSets->next);
  free(DisjointSets->setSize);
}

