/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_partitionOperations.h - bacis operations with partition structure
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/


#ifndef __PMORSY_PARTITION_OPERATIONS_H__
#define __PMORSY_PARTITION_OPERATIONS_H__

#include "morsy_types.h"
#include "morsy_routines.h"
#include "morsy_gainsOperations.h"

FLOAT_TYPE EvaluatePartitionWeighted(PartitionStructure* Partition,
                                     NestedDissectionParameters* params);

void InitializePartitionMemory(PartitionStructure* Partition, int n, 
                               int minGain, int maxGain, int nGains);

void InitializePartitionMemoryLight(PartitionStructure* Partition, int n, int isAdjToPartNeed);

void FreePartitionMemory(PartitionStructure* Partition);

void FreePartitionMemoryLight(PartitionStructure* Partition);

void CopyPartition(PartitionStructure* PartitionFrom, 
                   PartitionStructure* PartitionTo, int nVert);

void CopyPartitionParameters(PartitionStructure* PartitionFrom, 
                             PartitionStructure* PartitionTo);

int GetGainWeighted(PartitionStructure* Partition, int moveSep, 
                    int bigPart, GraphStructure* Graph);

int UpdatePartitionWeighted(int moveSep, PartitionStructure* Partition,  
                            GraphStructure* Graph, int part, 
                            int* MovedVertexes);

void GetAdjToPartWeights(PartitionStructure* Partition, GraphStructure* Graph);

void UpdateAdjToPartWeights(int moveSep, PartitionStructure* Partition,  
                            GraphStructure* Graph, int partToMove, 
                            int* MovedVertexes, int nMoved);

void CopyPartitionForList(PartitionStructure* PartitionFrom, 
                          PartitionStructure* PartitionTo, int* Vertexes,
                          int nVert, GraphStructure* Graph);

void CopyPartitionWithSepList(PartitionStructure* PartitionFrom, 
                   PartitionStructure* PartitionTo);

void ProjectAdjToPartWeights(GraphStructure* coarserGraph, GraphStructure* finerGraph, 
                             PartitionStructure* coarserPartition, 
                             PartitionStructure* finerPartition);

// EDGE PARTITIONS

int GetEdgeGain(PartitionStructure* Partition, int vertex);

void GetMinMaxEdgeGain(GraphStructure* Graph, int* minGain, int* maxGain);

void GetAdjToPartWeightsForEdgePartition(PartitionStructure* Partition, 
                                              GraphStructure* Graph);

void CopyAdjToPartWeights(PartitionStructure* PartitionFrom, 
                          PartitionStructure* PartitionTo);

void UpdateEdgePartitionAndGains(int movedVertex, PartitionStructure* Partition,  
                        GraphStructure* Graph);

void UpdateEdgePartitionAndGainsForBoundary(int movedVertex, 
                                            PartitionStructure* Partition,  
                                            GraphStructure* Graph, 
                                            int* isLooked);

void UpdateEdgePartitionAndGainsForAll(int movedVertex, 
                                            PartitionStructure* Partition,  
                                            GraphStructure* Graph, 
                                            int* isLooked);


#endif