/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_routines.h - common routines
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/


#ifndef __PMORSY_ROUTINES_H__
#define __PMORSY_ROUTINES_H__

#include "morsy_types.h"

#ifndef min
#define min(a, b) ((a)<(b))?(a):(b)
#endif
#ifndef max
#define max(a, b) ((a)>(b))?(a):(b)
#endif
#ifndef in
#define in(x, a, b) ((x)>=(a) && (x)<=(b)) ? (1) : (0)
#endif

int CheckPermutation(int* iperm, int n);

void GetPermFromIperm(int* iperm, int* perm, int n);

int LinearSearch(int* Vector, int key, int n);

int GetIntArrayMax(int* Vector, int n);

int GetIntArrayMin(int* Vector, int n);

void GetIntArrayMaxAndMin(int* Vector, int n, int* max, int* min);

int GetDoubleArrayMinIndex(FLOAT_TYPE* Vector, int n);

//-----------------------------------------------------------------------
//GRAPH COMPRESSION ROUTINES
//-----------------------------------------------------------------------
void SetNumbersUncompressed(int* iperm, int* Separators, int n);

void SetNumbersCompressed(int* iperm, int* Separators,
                          CompressionStructure* Compressed, int nSep, int n);
//-----------------------------------------------------------------------
//BASIC NESTED DISSECTION ROUTINES
//-----------------------------------------------------------------------
int FindSeparator(int vertex, int* GraphAdjncy, int* GraphXadj, 
                  int* VertexMarks, int* LevelStructure, int* LevelIndex,
                  int** Separator, int sepPos, int* nSep, int* nLevels);

int RootedLevelStructure(int root, int* GraphXadj, int* GraphAdjncy, 
                         int* VertexMarks, int* LevelStructure, 
                         int* LevelIndex);

int GetNewSubgraphs(int** Subgraphs, int* LevelStructure, int* LevelIndex, 
                    int level, int nSubgr, int nLevels);

int GetPartitionLevelSimple(int nLevels, int* LevelIndex);

FLOAT_TYPE EvaluatePartitionSimple(int nSep, int nFirstPart, int nSecondPart, int nVert);

int FindSeparatorWeighted(GraphStructure* Graph, int* LevelStructure, int* LevelIndex,
                          int* Separator, int* nSep);

int PartitionFromLevelStructure(int* LevelStructure, int* LevelIndex, 
                                int nLevels, PartitionStructure* Partition,
                                GraphStructure* Graph);

int VertexSeparatorFromEdgeSeparator(PartitionStructure* Partition,
                                      GraphStructure* Graph, int isAdjToPartWeightsUsed);

void GetEdgeCut(PartitionStructure* Partition,
                GraphStructure* Graph);

void InitializePartitionParameters(PartitionStructure* Partition, 
                                   GraphStructure* Graph);

void UpdateArrayForVertexesList(int* oldPartition, int* newPartition, 
                                int* movedVertexes, int nMoved);

#endif