/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_timers.h - timers for Windows and Linux
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/


#include "morsy_timer.h"

#ifdef _WIN32

void SetupTimer(WinTimer* timer)
{
  timer->sum = 0;
}

void StartTimer(WinTimer* timer)
{
  QueryPerformanceCounter((LARGE_INTEGER*)&(timer->start));
}

void FinishTimer(WinTimer* timer)
{
  QueryPerformanceCounter((LARGE_INTEGER*)&(timer->finish));
  timer->sum += timer->finish - timer->start;
}

void InitTimers(WinTimer* timers)
{
  int i;
  for (i = 0; i < N_TIMERS; i++)
  {
    timers[i].sum = 0;
  }
}

void FinilizeTimers(WinTimer* timers)
{
  LARGE_INTEGER freq;
  int i;

  QueryPerformanceFrequency(&freq);
  for (i = 0; i < N_TIMERS; i++)
  {
    timers[i].time = (double)timers[i].sum / (double)freq.QuadPart;
  }
}

#else

double getTimePoint()
{
  timeval t;
  gettimeofday(&t, NULL);
  return (double)t.tv_sec + (double)t.tv_usec * 1.0e-6;
}

void SetupTimer(MORSy_timer* timer)
{
  timer->sumD = 0.0;
  timer->time = 0.0;
}

void InitTimers(MORSy_timer* timers)
{
  int i;
  for (i = 0; i < N_TIMERS; i++)
  {
    timers[i].sumD = 0.0;
  }
}

void StartTimer(MORSy_timer* timer)
{
  timer->startD = getTimePoint();
}

void FinishTimer(MORSy_timer* timer)
{
  timer->finishD = getTimePoint();
  timer->sumD += timer->finishD - timer->startD;
}

void FinilizeTimers(MORSy_timer* timers)
{
  int i;
  for (i = 0; i < N_TIMERS; i++)
  {
    timers[i].time = timers[i].sumD;
  }
}





#endif

