/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* morsy_timers.h - timers declaration
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*/

#ifndef __PMORSY_TIMER__H__
#define __PMORSY_TIMER__H__

typedef enum
{
  MORSY_TIME_SUM,
  MORSY_TIME_COMPRESSION,
  MORSY_TIME_MATCHING,
  MORSY_TIME_CONTRACTING,
  MORSY_TIME_COARSENING,
  MORSY_TIME_INIT_PARTITIONING,
  MORSY_TIME_REFINEMENT,
  MORSY_TIME_PROJECTION,
  MORSY_TIME_UNCOARSENING,
  MORSY_TIME_SPLITTING,
  MORSY_TIME_LEAFS,
  MORSY_TIME_FIRST_TASK,
  MORSY_TIME_FIRST_TASK_COARSENING,
  MORSY_TIME_FIRST_TASK_MATCHING,
  MORSY_TIME_FIRST_TASK_INIT_PARTITIONING,
  MORSY_TIME_FIRST_TASK_REFINEMENT,
  MORSY_TIME_FIRST_TASK_UNCOARSENING,
} MORSY_TIMING_TYPE;

#define N_TIMERS 17

#ifdef _WIN32

#include "windows.h"

struct
{
  __int64 start;
  __int64 finish;
  __int64 freq;
  __int64 sum;
  double time;
} typedef WinTimer;

#define MORSy_timer WinTimer

#else

#include <time.h>
#include <sys/time.h>

typedef struct timeval timeval;

struct
{
  double startD;
  double finishD;
  double sumD;  
  timeval start;
  timeval finish;
  timeval sum;
  double time; 
} typedef GNUTimer;

#define MORSy_timer GNUTimer


#endif // WIN32

void SetupTimer(MORSy_timer* timer);
void InitTimers(MORSy_timer* timers);
void StartTimer(MORSy_timer* timer);
void FinishTimer(MORSy_timer* timer);
void FinilizeTimers(MORSy_timer* timers);

#endif
