/*
* Copyright (C) 2015 University of Nizhni Novgorod
* This file is part of the PMORSy ordering library, v 1.2.0
* 
* pmorsy.h - PMORSy' API
* 
* This file may be used under the terms of the GNU Lesser General Public 
* License version 2.1 as published by the Free Software Foundation 
* and appearing in the file LICENSE.LGPL included in the packaging of this 
* file. Please review the following information to ensure the GNU Lesser 
* General Public License version 2.1 requirements will be met: 
* http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
*
*/


#ifndef __PMORSY_H__
#define __PMORSY_H__

#define MORSY_PRINT
#define MORSY_TIMERS
//#define MORSY_USE_MKL

#define MORSY_PRECISION DOUBLE


#define MORSY_NUM_OPTIONS 15

typedef enum 
{
  MORSY_IS_DEFAULT, 
  MORSY_IS_COMPRESS,
  MORSY_MATCHING_TYPE,
  MORSY_PARTITIONING_TYPE,
  MORSY_REFINEMENT_TYPE,
  MORSY_COARSE_TO,
  MORSY_COARSERNING_STEPS,
  MORSY_DISBALANCE_COEFF,
  MORSY_REFINEMENT_STEP,
  MORSY_IS_CONNECTED,
  MORSY_IS_REFINEMENT_LIMITED,
  MORSY_NUM_THREADS,
  MORSY_REFINEMENT_STOP,
  MORSY_REFINEMENT_CUT,
  MORSY_BARRIER,
} morsyOptionNumbers;

typedef enum 
{
  MORSY_OK,
  MORSY_ERROR,
  MORSY_INPUT_ERROR,
  MORSY_MEMORY_ERROR,
  MORSY_COARSENING_ERROR,
  MORSY_PARTITIONING_ERROR,
  MORSY_REFINEMENT_ERROR,
} morsyErrorType;

/**
* API
*   int PMORSY_NestedDissection(int n, int* xadj, int* adjncy, 
*                        int* perm, int* iperm, int* options)
*   - parallel fill-reducing ordering 
* INPUT
*   int  n             - number of matrix rows or columns
*   int* xadj          - sparse matrix structure (or adjacency lists of the graph)
*   int* adjncy        
* OUTPUT
*   int* perm          - the inverse fill reducing ordering
*   int* iperm         - fill reducing ordering
* RETURN
*   error code
**/
int PMORSY_NestedDissection(int n, int* xadj, int* adjncy, int* perm, int* iperm, 
                     int* options);

/**
* API
*   void MORSY_SetDefaultOptions(int* options)
*   - to set default options
* INPUT
*   int  options       - the optioins array
*
**/
void MORSY_SetDefaultOptions(int* options);

/**
* API
*   void MORSY_SetTimePriorityOptions(int* options)
*   - setting options for minimizing the time of finding the ordering 
*     with the acceptable fill.
* INPUT
*   int  options       - the optioins array
*
**/
void MORSY_SetTimePriorityOptions(int* options);

/**
* API
*   void MORSY_SetQualityPriorityOptions(int* options)
*   - seting options to achieve the for minimizing number of non-zero elements
*     in the matrix factor
* INPUT
*   int  options       - the optioins array
*
**/
void MORSY_SetQualityPriorityOptions(int* options);

/**
* API
*   void MORSY_PrintOptions(int* options)
*   - print MORSy reordering options 
* INPUT
*   int  options       - the optioins array
*
**/
void MORSY_PrintOptions(int* options);


#endif