#include "reorderingRoutines.h"
#include "pmorsy.h"
#include "time.h"

int main(int argc, char** argv)
{
  int* adjncy;
  int* xadj;
  int* iperm;
  int* perm;
  int n;
  int nz;
  int error;
  int options[MORSY_NUM_OPTIONS];
  clock_t start, finish;
  double time;

  if (argc < 3)
  {
    printf("Command arguments:\n");
	printf("[1] - text file with matrix structure.\n");
    printf("    Format: \n      n nz\n");
    printf("    CRS columns array except diagonal elements\n");
    printf("    CRS rows array\n");
    printf("    Numbering starts from 0\n");
	printf("[2] - text file with permutation\n");
    return 1;
  }

  // read graph adjncy (matrix CRS columns) and graph adjncy index (matrix CRS rows)
  ReadGraphFromFile(argv[1], &adjncy, &xadj, &n, &nz);
  iperm = (int*)malloc(sizeof(int)*n);
   perm = (int*)malloc(sizeof(int)*n);
  // set ordering options
  MORSY_SetDefaultOptions(options);
  options[MORSY_IS_CONNECTED] = 1;
  options[MORSY_IS_REFINEMENT_LIMITED] = 0;
  options[MORSY_COARSE_TO] = 10; // only for small matrices
  //MORSY_PrintOptions(options);
  
  //call PMORSy  
  start = clock();
  error = PMORSY_NestedDissection(n, xadj, adjncy, perm, iperm, options);
  finish = clock();
  time = (double)(finish - start)/CLOCKS_PER_SEC;
  
  // print some information
  printf("is reordering done correctly: ");
  if (error)
  {
    printf("no\n");
  }
  else 
  {
    printf("yes\n");
  }
  printf("PMORSy reordering time: %.3f\n", time);
  
  WritePermutationToFile(argv[2], iperm, n);

  free(iperm);
  free(perm);
  free(adjncy);
  free(xadj);

  return 0;
}